// Copyright Sebastian Wiesner <sebastian@swsnr.de>
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.
//
// Alternatively, the contents of this file may be used under the terms
// of the GNU General Public License Version 2 or later, as described below:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
import Soup from "gi://Soup";
import { NotAnImageError, InvalidAPIKeyError } from "../source/errors.js";
import { encodeQuery } from "../network/uri.js";
import { HttpRequestError, HttpStatusError, downloadToFile, getJSON, } from "../network/http.js";
import metadata from "./metadata/apod.js";
/**
 * The APOD API did not return any body data.
 */
export class ApodErrorDataMissing extends Error {
    constructor() {
        super("Response returned no data");
    }
}
/**
 * The APOD API returned a non-200 response and provided a detailed error response.
 */
export class ApodError extends Error {
    code;
    constructor(code, message, options) {
        super(message, options);
        this.code = code;
    }
}
/**
 * Whether a value is a error response body from the APOD API.
 *
 * @param value The value to check
 * @returns true if value denotes an APOD error response, false otherwise
 */
const isApodErrorBody = (value) => {
    if (value && typeof value === "object" && Object.hasOwn(value, "error")) {
        const details = value["error"];
        return (details !== null &&
            typeof details === "object" &&
            typeof details["code"] === "string" &&
            typeof details["message"] === "string");
    }
    else {
        return false;
    }
};
/**
 * Query the metadata for todays image.
 *
 * @param session The HTTP session to use
 * @param apiKey The API key
 * @param cancellable The handle to cancel any IO operation
 * @returns The metadata for todays image.
 */
const queryMetadata = async (session, apiKey, cancellable) => {
    const query = [["api_key", apiKey]];
    const url = `https://api.nasa.gov/planetary/apod?${encodeQuery(query)}`;
    try {
        const response = await getJSON(session, url, cancellable);
        return response;
    }
    catch (error) {
        // Check if the API gave us a more specific response
        if (error instanceof HttpRequestError &&
            error.cause instanceof HttpStatusError &&
            error.cause.body) {
            let body = undefined;
            try {
                body = JSON.parse(new TextDecoder().decode(error.cause.body));
            }
            catch {
                body = undefined;
            }
            if (body && isApodErrorBody(body)) {
                // TODO: Inspect code and map to more precise errors
                throw new ApodError(body.error.code, body.error.message, {
                    cause: error,
                });
            }
            else {
                throw error;
            }
        }
        else {
            throw error;
        }
    }
};
/**
 * Create a downloader for APOD.
 *
 * @param settings The settings for this source.
 * @param directories Directories this source may use.
 * @returns A download function
 */
const createDownloader = (extensionMetadata, settings, directories) => {
    const version = extensionMetadata["version-name"] ?? "n/a";
    const session = new Soup.Session({
        user_agent: `${extensionMetadata.uuid}/${version} GNOME Shell extension`,
    });
    return async (cancellable) => {
        const apiKey = settings.get_string("api-key");
        if (apiKey === null || apiKey.length === 0) {
            throw new InvalidAPIKeyError(metadata);
        }
        const apodImageMetadata = await queryMetadata(session, apiKey, cancellable);
        const urlDate = apodImageMetadata.date.replaceAll("-", "").slice(2);
        const url = `https://apod.nasa.gov/apod/ap${urlDate}.html`;
        const imageMetadata = {
            title: apodImageMetadata.title,
            description: apodImageMetadata.explanation,
            url,
            copyright: apodImageMetadata.copyright ?? null,
        };
        if (apodImageMetadata.media_type !== "image") {
            throw new NotAnImageError(imageMetadata, apodImageMetadata.media_type);
        }
        const imageUrl = apodImageMetadata.hdurl ?? apodImageMetadata.url;
        const urlBasename = imageUrl.split("/").reverse()[0];
        const filename = urlBasename && 0 < urlBasename.length
            ? urlBasename
            : apodImageMetadata.title.replaceAll(/\/|\n/, "_");
        const targetFile = directories.imageDirectory.get_child(`${apodImageMetadata.date}-${filename}`);
        await downloadToFile(session, imageUrl, targetFile, cancellable);
        return {
            file: targetFile,
            metadata: imageMetadata,
        };
    };
};
/**
 * A source for images from APOD.
 */
export const source = {
    metadata,
    createDownloader,
};
export default source;
